#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#define BUFLEN 100

/* this routine read n bytes. It returns *0* if unable to read msgsize chars. */

static int readn (int sock, char *msgp, unsigned msgsize)
{
  int rlen, temp;

  rlen = 0;
  while(rlen < msgsize)
  {
    temp = read(sock, msgp+rlen, msgsize-rlen);
    if(temp<0)
      if(errno == EINTR)
	continue;
      else
	return 0;
    else if(temp == 0)
      break; /* End of file */

    rlen += temp;
  }
  
  
  return rlen==msgsize;
}


int main(int argc, char * argv[])
{
  int sockfd, i;
  struct sockaddr_in addr;
  char buf1[BUFLEN], buf2[BUFLEN];

  if (argc < 3) 
  {
    fprintf(stderr,"USAGE: %s ip_addr port.\n", argv[0]);
    exit(-1);
  }

  if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) 
  {
    perror("Can't get socket");
    exit(1);
  }

  memset(&addr, 0, sizeof(struct sockaddr_in));
  addr.sin_family = AF_INET;
  addr.sin_port = htons(atoi(argv[2]));
  inet_pton(AF_INET, argv[1], &addr.sin_addr);

  if (connect(sockfd, (struct sockaddr *)&addr, sizeof(addr)) < 0) 
  {
    perror("connect");
    exit(1);
  }

  i = 100;
  printf("Send %x\n", i);
  write(sockfd, &i, sizeof(i));
  
  while (fgets(buf1, BUFLEN, stdin)) /* Until EOF */
  {
    write(sockfd, buf1, strlen(buf1)+1);
    if (readn(sockfd, buf2, strlen(buf1)+1) == 0) 
    {
      fprintf(stderr, "Server closed connection.\n");
      exit(0);
    }
    printf("Server response : %s\n", buf2);
  }

  return 0;
}




